/*! 
* BeX5 v3 (http://www.justep.com) 
* Copyright 2015 Justep, Inc.
*/ 
define(function(require) {
	var $ = require("jquery");
	var justep = require("$UI/system/lib/justep");

	var Model = function() {
		this.callParent();
	};

/*
	入参数据结构
	{
		"filterData" : 过滤数据对象（bizData）  
		"filterCols" : 过滤列
		"exprID" : 表达式行id
		"exprObject" : {
			"col" : 选中关系的别名
			"operators" : 比较操作符数组
			"value" : 比较值
			"expr" : 表达式
			"exprKSQL" : 表达式
		}
	}
	返回数据结构
	{
		"exprID" : 
		"exprObject" : 
	}
 */
	Model.prototype.receiverReceive = function(event) {
		if (this.filterData != event.data.filterData || this.filterCols != event.data.filterCols) {
			this.filterData = event.data.filterData;
			this.filterCols = event.data.filterCols;
			this.comp("relationList").refresh(true);
		}

		this.exprID = event.data.exprID;
		this.exprObject = event.data.exprObject;

		this.init();
	};

	Model.prototype.relationDataCustomRefresh = function(event){
		var filterData = this.filterData;
		var filterCols = this.filterCols ? "," + this.filterCols + "," : this.filterCols;
		
		var rows = [];
		for ( var col in filterData.defCols) {
			var defCol = filterData.defCols[col];
			if (($.inArray(defCol.type, [ "String", "Integer", "Float", "Date", "Time", "DateTime", "Decimal", "Text" ]) == -1)
					|| (defCol.relation == "EXPRESS" || defCol.relation.substring(defCol.relation.length - 8) == ".version") 
					|| (filterCols && filterCols.indexOf("," + col + ",") == -1)) {
				continue;
			}

			var type;
			if (defCol.type == "Decimal") { 
				type = "Float"; 
			} else if (defCol.type == "Text") {
				type = "String";
			} else {
				type = defCol.type;
			}
			rows.push({
				"col" : col,
				"relation" : defCol.relation,
				"label" : defCol.label ? defCol.label : col,
				"type" : type,
				"typeLabel" : this.type2label(type)
			});
		}
		event.source.loadData({
			"rows" : rows,
			"@type" : "table"
		});	
	};

	Model.prototype.init = function() {
		var relationData = this.comp("relationData");
		var operatorData = this.comp("operatorData");
		var tempData = this.comp("tempData");

		$.each([ "NOT", "=", ">", "<", "LIKE", "IS NULL" ], function(i, n) {
			operatorData.setValueByID("checked", false, n);
		});
		$.each([ "stringValue", "integerValue", "floatValue", "dateValue", "timeValue", "datetimeValue" ], function(i, n) {
			tempData.setValue(n, null);
		});

		var row;
		if (this.exprObject && (row = relationData.getRowByID(this.exprObject.col)) !== null) {
			relationData.to(row);

			$.each(this.exprObject.operators, function(i, n) {
				operatorData.setValueByID("checked", true, n);
			});

			if (this.exprObject.value !== null) {
				var type = row.val("type");
				tempData.setValue(type.toLowerCase() + "Value", this.exprObject.value);
			}
		} else {
			relationData.first();
			operatorData.setValueByID("checked", true, "=");
		}
	};

	Model.prototype.type2label = function(type) {
		return {
			"String" : "字符串",
			"Integer" : "整型",
			"Float" : "数值",
			"Boolean" : "布尔",
			"Date" : "日期",
			"Time" : "时间",
			"DateTime" : "日期时间"
		}[type];
	};

	Model.prototype.operator2label = function(operator) {
		return {
			"NOT" : "不",
			"=" : "等于",
			">" : "大于",
			"<" : "小于",
			"LIKE" : "包含",
			"IS NULL" : "为空"
		}[operator];
	};

	Model.prototype.operatorBtnMousedown = function(event) {
		var operatorData = this.comp("operatorData");
		var operator = event.currentTarget.name;
		var checked = operatorData.getValueByID("checked", operator);
		operatorData.setValueByID("checked", !checked, operator);
	};

	Model.prototype.operatorBtnIsDisabled = function($element) {
		var relationData = this.comp("relationData");
		var operatorData = this.comp("operatorData");
		var disabled = relationData.val("type") != "String" && $element.name == "LIKE";
		if (disabled && operatorData.getValueByID("checked", $element.name)) {
			operatorData.setValueByID("checked", false, $element.name);
		}
		return disabled;
	};

	Model.prototype.operatorDataValueChange = function(event) {
		if (this._operatorDataValueChanging) {
			return;
		}
		this._operatorDataValueChanging = true;
		try {
			if (event.col == "checked" && event.newValue) {
				// 互斥
				var exclusive = {
					"NOT" : [],
					">" : [ "<", "LIKE", "IS NULL" ],
					"<" : [ ">", "LIKE", "IS NULL" ],
					"=" : [ "LIKE", "IS NULL" ],
					"LIKE" : [ "=", ">", "<", "IS NULL" ],
					"IS NULL" : [ "=", ">", "<", "LIKE" ]
				}[event.row.val("operator")];

				$.each(exclusive, function(i, n) {
					event.source.setValueByID("checked", false, n);
				});
			}
		} finally {
			this._operatorDataValueChanging = false;
		}
	};

	Model.prototype._getDateTimeFormat = function(format, type) {
		return {
			"chinese" : {
				"DateTime" : "yyyy年MM月dd日 hh时mm分ss秒",
				"Date" : "yyyy年MM月dd日",
				"Time" : "hh时mm分ss秒"
			},
			"sql" : {
				"DateTime" : "yyyy-MM-dd hh:mm:ss",
				"Date" : "yyyy-MM-dd",
				"Time" : "hh:mm:ss"
			},
			"standard" : {
				"DateTime" : justep.Date.STANDART_FORMAT,
				"Date" : justep.Date.STANDART_FORMAT_SHOT,
				"Time" : "hh:mm:ss"
			}
		}[format][type];
	};

	Model.prototype.datetime2str = function(datetime, type, format) {
		return justep.Date.toString(datetime, this._getDateTimeFormat(format, type));
	};

	Model.prototype.str2datetime = function(str, type, format) {
		return justep.Date.fromString(str, this._getDateTimeFormat(format, type));
	};

	Model.prototype.getSelectedRelationRow = function() {
		var relationRow = this.comp("relationData").getCurrentRow();
		if (!relationRow) {
			throw new Error("请选择过滤字段");
		}
		return relationRow;
	};

	Model.prototype.getSelectedOperators = function() {
		var operators = [];
		this.comp("operatorData").each(function(d) {
			if (d.row.val("checked")) {
				operators.push(d.row.val("operator"));
			}
		});
		if ((operators.length < 1) || (operators.length == 1 && operators[0] == "NOT")) {
			throw new Error("请选择有效的过滤操作");
		}
		return operators;
	};

	Model.prototype.getSelectedValue = function(relationRow, operators) {
		var tempData = this.comp("tempData");

		var type = relationRow.val("type");
		var value = "";
		if ($.inArray("IS NULL", operators) == -1) {
			value = tempData.getValue(type.toLowerCase() + "Value");
			if (typeof (value) == "undefined" || value === null || $.trim(value) === "") {
				throw new Error("请输入（或者选择）数据");
			}
			if (type == "String") {
				value = $.trim(value);
			} else if ((type == "Integer" || type == "Float") && (typeof (value) != "number")) {
				throw new Error("无效的数值数据");
			} else if ((type == "Date" || type == "Time" || type == "DateTime") && !(value instanceof Date)) {
				throw new Error("无效的日期时间数据");
			}
		}
		return value;
	};

	Model.prototype.transValueStr = function(relationRow, operators, value, isKSQL) {
		var type = relationRow.val("type");
		var valueStr = '';
		if ($.inArray("IS NULL", operators) != -1) {
			
		}
		else if (type == "String" && isKSQL) {
			if (($.inArray("LIKE", operators) != -1) && (value.indexOf("%") == -1)) {
				valueStr = "%" + value + "%";
			} else {
				valueStr = value;
			}
			valueStr = "'" + valueStr.replace(/'/g, "''") + "'";
		} else if ("DateTime".indexOf(type) != -1) {
			if (isKSQL) {
				valueStr = "'" + this.datetime2str(value, type, "sql") + "'";
			} else {
				valueStr = this.datetime2str(value, type, "chinese");
			}
		} else {
			valueStr = (typeof (value) == "undefined" || value === null) ? "" : value + "";
		}
		return valueStr;
	};

	Model.prototype.formatExpr = function(relationRow, operators, value, isKSQL) {
		var mainOperator = null;
		$.each(operators, function(i, n) {
			mainOperator = (n != "NOT" && n != "=") ? n : mainOperator;
		});

		var type = relationRow.val("type");
		var relationStr = !isKSQL ? relationRow.val("label") : ("DateTime".indexOf(type) == -1 ? relationRow.val("relation") : type + "ToString(" + relationRow.val("relation") + ")");
		var mainOperatorStr = (mainOperator === null) ? "" : (isKSQL ? mainOperator : this.operator2label(mainOperator));
		var valueStr = this.transValueStr(relationRow, operators, value, isKSQL);
		var notStr = ($.inArray("NOT", operators) == -1) ? "" : (isKSQL ? "NOT" : this.operator2label("NOT"));
		var equalStr = ($.inArray("=", operators) == -1) ? "" : (isKSQL ? "=" : this.operator2label("="));

		// {0} relation {1} mainOperator {2} value {3} NOT {4} =
		var exprFormat = isKSQL ? "{3} {0} {1}{4} {2}" : "{0} {3}{1}{4} {2}";
		return justep.String.format(exprFormat, relationStr, mainOperatorStr, valueStr, notStr, equalStr);
	};

	Model.prototype.getExprObject = function() {
		var relationRow = this.getSelectedRelationRow();
		var operators = this.getSelectedOperators();
		var value = this.getSelectedValue(relationRow, operators);
		var type = relationRow.val("type");
		return {
			"col" : relationRow.val("col"),
			"operators" : operators,
			"value" : ("DateTime".indexOf(type) != -1) ? this.datetime2str(value, type, "standard") : value,
			"expr" : this.formatExpr(relationRow, operators, value, false),
			"exprKSQL" : this.formatExpr(relationRow, operators, value, true)
		};
	};

	Model.prototype.valueIsDisabled = function() {
		return !this.comp("relationData").getCurrentRow() || this.comp("operatorData").getValueByID('checked', 'IS NULL');
	};

	Model.prototype.okBtnClick = function(event) {
		this.comp("receiver").windowEnsure({
			"exprID" : this.exprID,
			"exprObject" : this.getExprObject()
		});
	};

	Model.prototype.selectValueBtnClick = function(event) {
		var relationData = this.comp("relationData");
		this.comp("selectValueDialog").open({
			"data" : {
				"selectData" : this.filterData,
				"selectCol" : relationData.getValue("col"),
				"rangeFilter" : this.filterData.filters.toString()
			}
		});
	};

	Model.prototype.selectValueDialogReceive = function(event) {
		var type = this.comp("relationData").getValue("type");
		this.comp("tempData").setValueByID(type.toLowerCase() + "Value", event.data.value);
	};

	return Model;
});
